# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: spotimote.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='spotimote.proto',
  package='spotimote',
  serialized_pb=_b('\n\x0fspotimote.proto\x12\tspotimote\"\xcd\x01\n\tActionMsg\x12/\n\x08\x61\x63tionId\x18\x01 \x02(\x0e\x32\x1d.spotimote.ActionMsg.ActionId\"\x8e\x01\n\x08\x41\x63tionId\x12\x13\n\x0f\x41\x63tionPlayPause\x10\x01\x12\x0e\n\nActionNext\x10\x02\x12\x0e\n\nActionPrev\x10\x03\x12\x11\n\rActionShuffle\x10\x04\x12\x10\n\x0c\x41\x63tionRepeat\x10\x05\x12\x12\n\x0e\x41\x63tionVolumeUp\x10\x06\x12\x14\n\x10\x41\x63tionVolumeDown\x10\x07\"(\n\x0c\x42roadcastMsg\x12\n\n\x02ip\x18\x01 \x02(\t\x12\x0c\n\x04port\x18\x02 \x02(\x05\"D\n\x0e\x42roadcastReply\x12\n\n\x02id\x18\x01 \x02(\t\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\n\n\x02ip\x18\x03 \x02(\t\x12\x0c\n\x04port\x18\x04 \x02(\x05')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_ACTIONMSG_ACTIONID = _descriptor.EnumDescriptor(
  name='ActionId',
  full_name='spotimote.ActionMsg.ActionId',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ActionPlayPause', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ActionNext', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ActionPrev', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ActionShuffle', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ActionRepeat', index=4, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ActionVolumeUp', index=5, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ActionVolumeDown', index=6, number=7,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=94,
  serialized_end=236,
)
_sym_db.RegisterEnumDescriptor(_ACTIONMSG_ACTIONID)


_ACTIONMSG = _descriptor.Descriptor(
  name='ActionMsg',
  full_name='spotimote.ActionMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='actionId', full_name='spotimote.ActionMsg.actionId', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ACTIONMSG_ACTIONID,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=236,
)


_BROADCASTMSG = _descriptor.Descriptor(
  name='BroadcastMsg',
  full_name='spotimote.BroadcastMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ip', full_name='spotimote.BroadcastMsg.ip', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='port', full_name='spotimote.BroadcastMsg.port', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=238,
  serialized_end=278,
)


_BROADCASTREPLY = _descriptor.Descriptor(
  name='BroadcastReply',
  full_name='spotimote.BroadcastReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='spotimote.BroadcastReply.id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='spotimote.BroadcastReply.name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ip', full_name='spotimote.BroadcastReply.ip', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='port', full_name='spotimote.BroadcastReply.port', index=3,
      number=4, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=280,
  serialized_end=348,
)

_ACTIONMSG.fields_by_name['actionId'].enum_type = _ACTIONMSG_ACTIONID
_ACTIONMSG_ACTIONID.containing_type = _ACTIONMSG
DESCRIPTOR.message_types_by_name['ActionMsg'] = _ACTIONMSG
DESCRIPTOR.message_types_by_name['BroadcastMsg'] = _BROADCASTMSG
DESCRIPTOR.message_types_by_name['BroadcastReply'] = _BROADCASTREPLY

ActionMsg = _reflection.GeneratedProtocolMessageType('ActionMsg', (_message.Message,), dict(
  DESCRIPTOR = _ACTIONMSG,
  __module__ = 'spotimote_pb2'
  # @@protoc_insertion_point(class_scope:spotimote.ActionMsg)
  ))
_sym_db.RegisterMessage(ActionMsg)

BroadcastMsg = _reflection.GeneratedProtocolMessageType('BroadcastMsg', (_message.Message,), dict(
  DESCRIPTOR = _BROADCASTMSG,
  __module__ = 'spotimote_pb2'
  # @@protoc_insertion_point(class_scope:spotimote.BroadcastMsg)
  ))
_sym_db.RegisterMessage(BroadcastMsg)

BroadcastReply = _reflection.GeneratedProtocolMessageType('BroadcastReply', (_message.Message,), dict(
  DESCRIPTOR = _BROADCASTREPLY,
  __module__ = 'spotimote_pb2'
  # @@protoc_insertion_point(class_scope:spotimote.BroadcastReply)
  ))
_sym_db.RegisterMessage(BroadcastReply)


# @@protoc_insertion_point(module_scope)



from __future__ import print_function
import time
import socket
import argparse
import netifaces

import requests

import spotimote_pb2


MULTICAST_GROUP = '224.242.224.224'
MULTICAST_PORT = 5116
MULTICAST_TIMEOUT = 5
COMMANDS = {}


def command(function):
    COMMANDS[function.__name__] = function


class Spotimote(object):
    def __init__(self, host=None, port=MULTICAST_PORT):
        self.host = host
        self.port = port

    def get_host(self):
        host = getattr(self, '_host', None)
        if not host:
            _, host, port = self.detect().next()
            self.host = host
            self.port = port

        return host

    def set_host(self, host):
        self._host = host

    def del_host(self):
        self._host = None

    host = property(get_host, set_host, del_host,
                    'The Spotimote host, will be automatically detected if not given')

    def _send_broadcast(self):
        broadcaster = socket.socket(socket.AF_INET, socket.SOCK_DGRAM,
                                    socket.IPPROTO_UDP)
        broadcaster.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 32)

        broadcast_message = spotimote_pb2.BroadcastMsg()
        broadcast_message.port = MULTICAST_PORT
        for interface in netifaces.interfaces():
            addresses = netifaces.ifaddresses(interface)
            for address in addresses.get(netifaces.AF_INET, []):
                if 'netmask' in address:
                    broadcast_message.ip = address['addr']
                    broadcaster.sendto(broadcast_message.SerializeToString(),
                                       (MULTICAST_GROUP, MULTICAST_PORT))

    def detect(self):
        listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            listener.settimeout(MULTICAST_TIMEOUT)
            listener.bind(('', MULTICAST_PORT))
            listener.listen(5)

            self._send_broadcast()

            start_time = time.time()
            while time.time() - start_time < MULTICAST_TIMEOUT:
                try:
                    connection, client_address = listener.accept()
                except socket.timeout:
                    break

                try:
                    data = connection.recv(1024)
                finally:
                    connection.close()

                broadcast_reply = spotimote_pb2.BroadcastReply()
                broadcast_reply.ParseFromString(data)
                yield broadcast_reply.name, broadcast_reply.ip,\
                      broadcast_reply.port
        finally:
            listener.close()

    def action(self, action_id):
        action = spotimote_pb2.ActionMsg()
        action.actionId = action_id
        requests.post('http://%s:5116/action' % self.host,
                      action.SerializeToString(), timeout=0.5)

    @command
    def play_pause(self):
        return self.action(spotimote_pb2.ActionMsg.ActionPlayPause)

    @command
    def next(self):
        return self.action(spotimote_pb2.ActionMsg.ActionNext)

    @command
    def prev(self):
        return self.action(spotimote_pb2.ActionMsg.ActionPrev)

    @command
    def shuffle(self):
        return self.action(spotimote_pb2.ActionMsg.ActionShuffle)

    @command
    def repeat(self):
        return self.action(spotimote_pb2.ActionMsg.ActionRepeat)

    @command
    def volume_up(self):
        return self.action(spotimote_pb2.ActionMsg.ActionVolumeUp)

    @command
    def volume_down(self):
        return self.action(spotimote_pb2.ActionMsg.ActionVolumeDown)


def main():
    parser = argparse.ArgumentParser(description='Control Spotimote')
    parser.add_argument('--host', help='The hostname or IP of the Spotimote '
                                       'server. If no host is given it will '
                                       'be detected automatically.')
    parser.add_argument('--port', default=MULTICAST_PORT,
                        help='The port of the Spotimote server')
    parser.add_argument('command', metavar='COMMAND', choices=COMMANDS.keys(),
                        help='Choose a command to execute (%s)' % ', '.join(
                            COMMANDS.keys()))

    try:
        args = parser.parse_args()
    except SystemExit:
        parser.print_help()
        raise

    spotimote = Spotimote()
    command = COMMANDS[args.command]

    if args.host:
        hosts = [('Commandline', args.host, args.port), ]
    else:
        print('Detecting hosts')
        hosts = list(spotimote.detect())
        if hosts:
            print('Got hosts:')
            for host in hosts:
                print(host)
        else:
            print('No hosts found')

    for _, host, port in hosts:
        print('Executing %s on %s:%s' % (args.command, host, port))
        spotimote.host = host
        spotimote.port = port
        command(spotimote)


if __name__ == '__main__':
    main()




__package_name__ = 'spotimote'
__import_name__ = 'spotimote'
__version__ = '1.0.1'
__author__ = 'Rick van Hattem'
__author_email__ = 'Wolph@Wol.ph'
__description__ = '''
Library to control Spotimote from Python
'''
__url__ = 'https://github.com/WoLpH/spotimote'

